"""
This type stub file was generated by pyright.
"""

def json_serial(o):  # -> str | float | int | list[Any]:
    ...
def dumps(dct, as_dict=..., **kwargs):  # -> object | str:
    """Dump the dict to json in DynamoDB Format
    You can use any other simplejson or json options
    :param dct - the dict to dump
    :param as_dict - returns the result as python dict (useful for DynamoDB boto3 library) or as json sting
    :returns: DynamoDB json format.
    """
    ...

def object_hook(dct):  # -> datetime | str | list[Any] | float | int | set[Any] | None:
    """DynamoDB object hook to return python values"""
    ...

def loads(s: str | dict, as_dict: bool = False, *args, **kwargs):  # -> Any:
    """Loads dynamodb json format to a python dict.
    :param s - the json string or dict (with the as_dict variable set to True) to convert
    :returns python dict object
    """
    ...
